<?php


namespace Bbg\ProductAssets\Assets;


use Bbg\ProductAssets\CDN\AbstractCDN;
use Bbg\ProductAssets\Collection\BeforeAfterCollection;
use Bbg\ProductAssets\Collection\ImageCollection;
use Bbg\ProductAssets\Collection\ThreeHundredSixtyCollection;
use Bbg\ProductAssets\Collection\VideoCollection;
use DateTime;
use Exception;
use stdClass;


abstract class AbstractAssets
{
    const URL_SEPARATOR = '/';
    const URL_JSONFILE = 'asset.json';
    const IMAGE_TYPE_PRIMARY = 'primary';
    const IMAGE_TYPE_SECONDARY = 'secondary';
    const IMAGE_TYPE_VARIATIONS = 'variations';
    const IMAGE_TYPE_INUSE = 'inuse';
    const IMAGE_TYPE_STYLEDSCENES = 'styledscenes';
    const IMAGE_TYPE_TEXTURE = 'texture';
    const IMAGE_TYPE_WITHTEXT = 'withtext';
    const IMAGE_TYPE_WITHLICENSE = 'withlicense';
    const IMAGE_TYPES = [
        self::IMAGE_TYPE_PRIMARY,
        self::IMAGE_TYPE_SECONDARY,
        self::IMAGE_TYPE_VARIATIONS,
        self::IMAGE_TYPE_INUSE,
        self::IMAGE_TYPE_STYLEDSCENES,
        self::IMAGE_TYPE_TEXTURE,
        self::IMAGE_TYPE_WITHTEXT,
        self::IMAGE_TYPE_WITHLICENSE,
    ];
    const VIDEO_TYPE_APPLICATION = 'application';
    const VIDEO_TYPE_MARKETING = 'marketing';
    const VIDEO_TYPES = [
        self::VIDEO_TYPE_APPLICATION,
        self::VIDEO_TYPE_MARKETING,
    ];
    private string $sku;
    private stdClass $assetObject;
    private ?AbstractCDN $cdn;

    public function __construct(string $sku, stdClass $assetObject, ?AbstractCDN $cdn = null)
    {
        $this
            ->setSku($sku)
            ->setAssetObject($assetObject);

        if (!is_null($cdn)) {
            $this
                ->setCDN($cdn);
        }
    }

    public static function getJSONPath(string $sku): string
    {
        return implode(self::URL_SEPARATOR, array(self::getRootPath($sku), self::URL_JSONFILE));
    }

    public static function getRootPath(string $sku): string
    {
        if (strlen($sku) >= 4) {
            return implode(self::URL_SEPARATOR, array(null, 'product', $sku[0], $sku[1], $sku[2], $sku[3], $sku));
        }

        return self::URL_SEPARATOR;
    }

    public function hasBeforeAfter(): bool
    {
        return $this->has('beforeAfter');
    }

    protected function has(string $section): bool
    {
        return ((isset($this->assetObject->{$section})) && (isset($this->assetObject->{$section}->globals)) && ((is_array($this->assetObject->{$section}->globals)) || (is_object($this->assetObject->{$section}->globals))));
    }

    public function hasBeforeAfterLocale(?string $locale = null): bool
    {
        return $this->hasLocale('beforeAfter', $locale);
    }

    protected function hasLocale(string $section, ?string $locale = null): bool
    {
        $locale = self::validateLocale($locale);

        if (is_null($locale)) {
            return ((isset($this->assetObject->{$section})) && (isset($this->assetObject->{$section}->locales)) && (is_object($this->assetObject->{$section}->locales)));
        }

        return ((isset($this->assetObject->{$section})) && (isset($this->assetObject->{$section}->locales)) && (isset($this->assetObject->{$section}->locales->{$locale})) && ((is_array($this->assetObject->{$section}->locales->{$locale})) || (is_object($this->assetObject->{$section}->locales->{$locale}))));
    }

    protected static function validateLocale(?string $locale = null): ?string
    {
        if (is_null($locale)) {
            return $locale;
        }

        return (preg_match('/^[a-z]{2}_[A-Z]{2}$/', $locale) || preg_match('/^[a-z]{2}$/', $locale) ? $locale : null);
    }

    public function getBeforeAfterLocales(): array
    {
        return $this->getLocales('beforeAfter');
    }

    protected function getLocales(string $section): array
    {
        $locales = array();

        if ((isset($this->assetObject->{$section})) && (isset($this->assetObject->{$section}->locales)) && (is_object($this->assetObject->{$section}->locales))) {
            foreach ($this->assetObject->{$section}->locales as $locale => $data) {
                if (self::validateLocale($locale) === $locale) {
                    $locales[] = $locale;
                }
            }
        }

        return $locales;
    }

    public function getBeforeAfter(?string $locale = null): BeforeAfterCollection
    {
        $locale = self::validateLocale($locale);

        if (isset($this->assetObject->beforeAfter)) {
            if ((!is_null($locale)) && (isset($this->assetObject->beforeAfter->locales)) && (isset($this->assetObject->beforeAfter->locales->{$locale}))) {
                return new BeforeAfterCollection($this->assetObject->beforeAfter->locales->{$locale}, $this->getPath(), $this->getCDN());
            }

            if (isset($this->assetObject->beforeAfter->globals)) {
                return new BeforeAfterCollection($this->assetObject->beforeAfter->globals, $this->getPath(), $this->getCDN());
            }
        }

        return new BeforeAfterCollection(array(), $this->getPath());
    }

    public function getPath(): string
    {
        if (isset($this->assetObject->path)) {
            return (string)$this->assetObject->path;
        }

        return self::URL_SEPARATOR;
    }

    public function getCDN(): ?AbstractCDN
    {
        return $this->cdn;
    }

    public function setCDN(AbstractCDN $cdn): AbstractAssets
    {
        $this->cdn = $cdn;

        return $this;
    }

    public function hasImage(): bool
    {
        return $this->has('image');
    }

    public function hasImageLocale(?string $locale = null): bool
    {
        return $this->hasLocale('image', $locale);
    }

    public function getImageLocales(): array
    {
        return $this->getLocales('image');
    }

    public function hasImageGroup(?string $locale, string $group): bool
    {
        return $this->hasGroup('image', $locale, $group);
    }

    protected function hasGroup(string $section, ?string $locale, string $group): bool
    {
        $locale = self::validateLocale($locale);
        $group = (in_array($group, self::IMAGE_TYPES) || in_array($group, self::VIDEO_TYPES) ? $group : null);

        $dataObject = $this->getObject($locale, $section);

        return ((isset($dataObject->{$group})) && (is_array($dataObject->{$group})));
    }

    /**
     * @param string|null $locale
     * @param string $section
     * @return mixed
     */
    protected function getObject(?string $locale, string $section): mixed
    {
        if ((!is_null($locale)) && (isset($this->assetObject->{$section}->locales)) && (isset($this->assetObject->{$section}->locales->{$locale}))) {
            $dataObject = $this->assetObject->{$section}->locales->{$locale};
        } else {
            $dataObject = $this->assetObject->{$section}->globals;
        }

        return $dataObject;
    }

    public function getImageGroups(?string $locale = null): array
    {
        return $this->getGroups('image', $locale);
    }

    protected function getGroups(string $section, ?string $locale = null): array
    {
        $locale = self::validateLocale($locale);
        $groups = array();

        $dataObject = $this->getObject($locale, $section);

        if ((isset($dataObject)) && (is_object($dataObject))) {
            foreach ($dataObject as $group => $data) {
                $groups[] = $group;
            }
        }

        return $groups;
    }

    public function getImage(?string $type = null, ?string $locale = null): ImageCollection
    {
        $type = (in_array($type, self::IMAGE_TYPES) ? $type : self::IMAGE_TYPE_PRIMARY);
        $locale = self::validateLocale($locale);

        if (isset($this->assetObject->image)) {
            if ((!is_null($locale)) && (isset($this->assetObject->image->locales)) && (isset($this->assetObject->image->locales->{$locale}))) {
                $dataObject = $this->assetObject->image->locales->{$locale};
            } else {
                $dataObject = $this->assetObject->image->globals;
            }

            if (isset($dataObject->{$type})) {
                return new ImageCollection($dataObject->{$type}, $this->getPath(), $this->getCDN());
            }
        }

        return new ImageCollection(array(), $this->getPath());
    }

    public function hasThreeHundredSixty(): bool
    {
        return ((isset($this->assetObject->threeHundredSixty)) && (is_array($this->assetObject->threeHundredSixty)));
    }

    public function getThreeHundredSixty(?int $collectionIndex = null): ThreeHundredSixtyCollection
    {
        $collectionIndex = (is_null($collectionIndex) ? 0 : $collectionIndex);

        if ((isset($this->assetObject->threeHundredSixty)) && (is_array($this->assetObject->threeHundredSixty))) {
            if ((isset($this->assetObject->threeHundredSixty[$collectionIndex])) && (is_array($this->assetObject->threeHundredSixty[$collectionIndex]))) {
                return new ThreeHundredSixtyCollection($this->assetObject->threeHundredSixty[$collectionIndex], $this->getPath(), $this->getCDN());
            }
        }

        return new ThreeHundredSixtyCollection(array(), $this->getPath());
    }

    public function countThreeHundredSixty(): int
    {
        if ((isset($this->assetObject->threeHundredSixty)) && (is_array($this->assetObject->threeHundredSixty))) {
            return count($this->assetObject->threeHundredSixty);
        }

        return 0;
    }

    public function hasVideo(): bool
    {
        return $this->has("video");
    }

    public function hasVideoLocale(?string $locale = null): bool
    {
        return $this->hasLocale('video', $locale);
    }

    public function getVideoLocales(): array
    {
        return $this->getLocales('video');
    }

    public function hasVideoGroup(?string $locale, string $group): bool
    {
        return $this->hasGroup('video', $locale, $group);
    }

    public function getVideoGroups(?string $locale = null): array
    {
        return $this->getGroups('video', $locale);
    }

    public function getVideo(?string $type = null, ?string $locale = null): VideoCollection
    {
        $type = (in_array($type, self::VIDEO_TYPES) ? $type : self::IMAGE_TYPE_PRIMARY);
        $locale = self::validateLocale($locale);

        if (isset($this->assetObject->video)) {
            if ((!is_null($locale)) && (isset($this->assetObject->video->locales)) && (isset($this->assetObject->video->locales->{$locale}))) {
                $dataObject = $this->assetObject->video->locales->{$locale};
            } elseif (isset($this->assetObject->video->globals)) {
                $dataObject = $this->assetObject->video->globals;
            }

            if (isset($dataObject->{$type})) {
                return new VideoCollection($dataObject->{$type}, $this->getPath(), $this->getCDN());
            }
        }

        return new VideoCollection(array(), $this->getPath());
    }

    public function getSku(): string
    {
        return $this->sku;
    }

    public function setSku(string $sku): AbstractAssets
    {
        $this->sku = $sku;

        return $this;
    }

    public function getAssetObject(): stdClass
    {
        return $this->assetObject;
    }

    public function setAssetObject(stdClass $assetObject): AbstractAssets
    {
        $this->assetObject = $assetObject;

        return $this;
    }

    /**
     * @throws Exception
     */
    public function getJSONCacheDate(): DateTime
    {
        if (isset($this->assetObject->cached)) {
            return new DateTime($this->assetObject->cached);
        }

        return new DateTime();
    }
}



